'From Squeak3.9 of 7 November 2006 [latest update: #7067] on 29 March 2008 at 9:35:46 pm'!
"Change Set:		FFI-64BitUpdates-ExternalTypes-dtl
Date:			29 March 2008
Author:			David T. Lewis

Updates to external address representations to enable FFI on 64-bit hosts and with 64-bit images."!

ByteArray variableByteSubclass: #ExternalAddress
	instanceVariableNames: ''
	classVariableNames: 'PointerSize'
	poolDictionaries: ''
	category: 'FFI-Kernel'!

!ByteArray methodsFor: '*FFI-Kernel' stamp: 'dtl 1/6/2008 23:52'!
asExternalPointer
	"Convert the receiver assuming that it describes a pointer to an object."
	| externalAddress |
	externalAddress := ExternalAddress new.
	(1 to: self size) do: [:i | externalAddress basicAt: i put: (self byteAt: i)].
	^externalAddress! !


!ExternalAddress methodsFor: 'private' stamp: 'dtl 1/6/2008 23:14'!
asByteArrayPointer
	"Return a ByteArray describing a pointer to the contents of the receiver."
	| byteArray |
	byteArray := ByteArray new: self size.
	(1 to: self size) do: [:i | byteArray byteAt: i put: (self basicAt: i)].
	^byteArray! !

!ExternalAddress methodsFor: 'converting' stamp: 'dtl 1/6/2008 23:46'!
asInteger
	"convert address to integer"
	| byteArray hi lo |
	byteArray := self asByteArrayPointer.
	self size = 4
		ifTrue: [^ byteArray unsignedLongAt: 1].
	self size = 8
		ifTrue: [
			hi := byteArray unsignedLongAt: 1.
			lo := byteArray unsignedLongAt: 5.
			^ hi * 16r1000000 + lo]
! !


!ExternalAddress class methodsFor: 'instance creation' stamp: 'dtl 1/6/2008 23:04'!
new
	"External addresses are typically either 4 or 8 bytes long, depending
	on the runtime platform."
	^super new: PointerSize! !

!ExternalAddress class methodsFor: 'accessing' stamp: 'dtl 3/22/2008 18:14'!
pointerSize
	"Answer pointer size. See #sizeOfPointer to obtain value directly from platform."
	^PointerSize! !

!ExternalAddress class methodsFor: 'class initialization' stamp: 'dtl 3/24/2008 21:13'!
initialize
	"ExternalAddress initialize"

	PointerSize := self sizeOfPointer.
	ExternalType initialize. "ExternalType uses #pointerSize".
	ExternalStructure compileAllFields. "Uses ExternalType"! !

!ExternalAddress class methodsFor: 'class initialization' stamp: 'dtl 3/24/2008 21:13'!
startUp: resuming 
	resuming ifTrue: [self allInstancesDo: [:addr | addr beNull].
			self initialize]! !

!ExternalAddress class methodsFor: 'platform dependent' stamp: 'dtl 1/6/2008 22:53'!
sizeOfPointer
	"Size of a void pointer for this C compiler on this machine."

	<primitive: 'primitiveSizeOfPointer' module: 'SqueakFFIPrims'>
	^ 4 "default to 32 bit pointers"
! !


!ExternalType class methodsFor: 'private' stamp: 'dtl 3/22/2008 13:38'!
pointerSpec
	^(ExternalAddress pointerSize bitOr: FFIFlagPointer)
! !

ExternalAddress initialize!

!ExternalAddress class reorganize!
('instance creation' allocate: gcallocate: new new:)
('accessing' pointerSize)
('class initialization' initialize startUp:)
('platform dependent' sizeOfPointer)
!

ExternalAddress removeSelector: #fromInteger:!
